unit datfile;

interface
uses SysUtils, Classes, Misc, dialogs, BZip2;

type
  TFileNfo=packed record
    fn: string[16];
    size: integer;
    offset: LongInt;
  end;

procedure Save(fn:string; fl:TStringList; DeleteOriginal: boolean);
procedure SaveToEnd(fn, src:string; DeleteOriginal: boolean); //     // fn- , fl- ,  
procedure Load(fl, fn:string; var s:TMemoryStream);
procedure LoadUc(fl, fn:string; var s:TMemoryStream);
procedure LoadCompFromStream(Stream: TStream; var s:TMemoryStream);
procedure LoadFromEnd(fl:string; var s:TMemoryStream);
function EncDec(str,key: string): string;

implementation

function EncDec(str,key: string): string;
var
 i: integer;
 iKey: integer;
begin
 Result:='';
 iKey:=StrToInt(key);
 for i:=1 to Length(str) do
 begin
  if i mod 2 = 0 then inc(iKey)
  else dec(iKey);
  Result:= Result + Chr(Ord(str[i]) XOR iKey);
 end;
end;

procedure LoadCompFromStream(Stream: TStream; var s:TMemoryStream);
const
  BufferSize = 65536;
var
  DecompressionStream: TStream;
  Count: Integer;
  Buffer: array[0..BufferSize - 1] of Byte;
begin
  try
    DecompressionStream := TBZDecompressionStream.Create(Stream);
    try
      while True do
      begin
        Count := DecompressionStream.Read(Buffer, BufferSize);
        if Count <> 0 then s.WriteBuffer(Buffer, Count) else Break;
      end;
    finally
      DecompressionStream.Free;
    end;
    s.Position := 0;
  finally
  end;
end;

procedure Save(fn:string; fl:TStringList; DeleteOriginal: boolean); //     // fn- , fl- ,  
var
  k,i:  integer;
  f:    TFileStream;
  s:    file;
  o:    integer;
  r:    TFileNfo;
  p:    ^byte;
  t:    string;
begin
  f:=TFileStream.Create(fn,fmCreate);
  k:=fl.Count;
  f.Write(k,SizeOf(k)); //   
  o:=SizeOf(r)*k+SizeOf(k); //   
  if k>0 then
   for i:=0 to k-1 do
   begin //  
    Assign(s,fl[i]); Reset(s,1);
    t:= ExtractFileName(fl[i]);
    r.fn:=EncDec(t,IntToStr(Ord(t[1])));
    r.size:=FileSize(s);
    r.offset:=o; o:=o+r.size;
    f.Write(r,SizeOf(r));
    t:='';
    CloseFile(s);
   end;
  if k>0 then
   for i:=0 to k-1 do
   begin //  
    Assign(s,fl[i]); Reset(s,1);
    r.size:=FileSize(s);
    GetMem(p,r.size);
    BlockRead(s,p^,r.size,r.offset);
    f.Write(p^,r.size);
    FreeMem(p,r.size);
    CloseFile(s);
    if DeleteOriginal then DeleteFile(fl[i]);
   end;
  f.Free;
end;


procedure SaveToEnd(fn, src:string; DeleteOriginal: boolean); //     // fn- , fl- ,  
var
  f:    TFileStream;
  s:    file;
  o:    integer;
  p:    ^byte;
begin
  f:=TFileStream.Create(fn,fmOpenReadWrite);
  f.Seek(0, soFromEnd);
  Assign(s,src); Reset(s,1);
  o:=FileSize(s);
  GetMem(p,o);
  BlockRead(s,p^,o);
  f.Write(p^,o);
  FreeMem(p,o);
  f.Write(o,4);
  f.Free;
end;


procedure Load(fl, fn:string; var s:TMemoryStream); overload; //     
var
  k,i:    integer;
  f:      TFileStream;
  p:      ^byte;
  r:      TFileNfo;
  t:      TMemoryStream;
begin
  f:=TFileStream.Create(fl,fmOpenRead);
  s.Clear;
  t:=TMemoryStream.Create;
  f.Read(k,SizeOf(k)); i:=0; //   
  while i<k do
  begin //       
    f.ReadBuffer(r,SizeOf(r));
    if r.fn=EncDec(fn,IntToStr(Ord(fn[1]))) then i:=k; //        
    inc(i);
  end;
  if i>k then
  begin //    -  
    f.Seek(r.offset,soFromBeginning);
    GetMem(p,r.size);
    f.ReadBuffer(p^,r.size);
    t.WriteBuffer(p^,r.size); //    
    FreeMem(p,r.size);
  end;
  f.Free;  t.Position:=0;
  LoadCompFromStream(t,s);
  t.Free;
end;


procedure LoadUc(fl, fn:string; var s:TMemoryStream); //     
var
  k,i:    integer;
  f:      TFileStream;
  p:      ^byte;
  r:      TFileNfo;
begin
  f:=TFileStream.Create(fl,fmOpenRead);
  s.Clear;
  f.Read(k,SizeOf(k)); i:=0; //   
  while i<k do
  begin //       
    f.ReadBuffer(r,SizeOf(r));
    if r.fn=fn then i:=k; //        
    inc(i);
  end;
  if i>k then
  begin //    -  
    f.Seek(r.offset,soFromBeginning);
    GetMem(p,r.size);
    f.ReadBuffer(p^,r.size);
    s.WriteBuffer(p^,r.size); //    
    FreeMem(p,r.size);
  end;
  f.Free;  s.Position:=0;
end;


procedure LoadFromEnd(fl:string; var s:TMemoryStream);  //     
var
  k,i:    integer;
  f:      TFileStream;
  p:      ^byte;
  r:      TFileNfo;
  t:      TMemoryStream;
begin
  f:=TFileStream.Create(fl,fmOpenRead);
  s.Clear;
  t:=TMemoryStream.Create;
  f.Seek(f.Size - SizeOf(k),soFromBeginning);
  f.Read(k,SizeOf(k));
  f.Seek(f.Size - SizeOf(k) - k ,soFromBeginning);
  GetMem(p,k);
  f.ReadBuffer(p^,k);
  t.WriteBuffer(p^,k);
  FreeMem(p,k);
  f.Free;  t.Position:=0;
  LoadCompFromStream(t,s);
  t.Free;
end;


end.
