unit LstFile;

interface
uses
  Classes, SysUtils, Misc;

type TListFile = class(TObject)
  private
    LFile: textfile;
    ListStrings: TStringList;
    CreatedFromStream: boolean;
  public
    constructor Create(FName: string);
    constructor CreateFromStream(Stream: TStream);    
    destructor Destroy;
    function GetFSection(SName : string) : TStringList;
    function GetKeyValue(SName, Key : string) : string;
    function GetKeyValueFromList(SName, Key : string) : string;
    function GetSectionFromList(SName : string) : TStringList;
  end;


implementation

constructor TListFile.Create(FName: string);
begin
  inherited Create;
  AssignFile(LFile,FName);
  CreatedFromStream:=false;  
end;

constructor TListFile.CreateFromStream(Stream: TStream);
begin
  inherited Create;
  ListStrings:=TStringList.Create;
  ListStrings.LoadFromStream(Stream);
  CreatedFromStream:=true;
end;

function TListFile.GetFSection(SName : string) : TStringList;
var
  TS : string;
begin
  Result:=TStringList.Create;
  Reset(LFile);
  while not Eof(LFile) do
  begin
    ReadLn(LFile,TS);
    TS:=Trim(TS);
    if (TS = '['+SName+']') then
    begin
      repeat
        ReadLn(LFile,TS);
        TS:=Trim(TS);
        if(TS<>'') then Result.Add(TS);
      until (TS='[/'+SName+']');
      if(Result.Count>0) then Result.Delete(Result.Count-1);
      Break;
    end;
  end;
  if(Result.Count=0) then Result.Add('FAIL');
end;

function TListFile.GetKeyValue(SName, Key : string) : string;
var
  TS : string;
  Params : TStringList;
begin
  Reset(LFile);
  Result:='';
  Params:=TStringList.Create;
  while not Eof(LFile) do
  begin
    ReadLn(LFile,TS);
    TS:=Trim(TS);
    if (TS = '['+SName+']') then
    begin
      repeat
        ReadLn(LFile,TS);
        TS:=Trim(TS);
        if(TS<>'') then
        begin
          Params:=Tokenize(TS,'|');
          if(Params[Params.Count-1]=Key) then
          begin
            Result:=Params[0];
            break;
          end;
        end;
      until (TS='[/'+SName+']');
      Break;
    end;
  end;
end;

function TListFile.GetSectionFromList(SName : string) : TStringList;
var
  TS : string;
  i  : integer;
begin
  Result:=TStringList.Create;
  i:=0;
  while i<ListStrings.Count do
  begin
    TS:=Trim(ListStrings[i]);
    if (TS = '['+SName+']') then
    begin
      repeat
        inc(i);
        TS:=Trim(ListStrings[i]);
        if(TS<>'') then Result.Add(TS);
      until (TS='[/'+SName+']') or (i = ListStrings.Count);
      if(Result.Count>0) then Result.Delete(Result.Count-1);
      Break;
    end;
   inc(i);
  end; // end of while
  if(Result.Count=0) then Result.Add('FAIL');
end;

function TListFile.GetKeyValueFromList(SName, Key : string) : string;
var
  TS : string;
  Params : TStringList;
  i: integer;
begin
  Result:='';
  Params:=TStringList.Create;
  i:=0;
  if ListStrings.Count=0 then exit;
  while i<ListStrings.Count do
  begin
    TS:=Trim(ListStrings[i]);
    if (TS = '['+SName+']') then
    begin
      repeat
        inc(i);
        TS:=Trim(ListStrings[i]);
        if(TS<>'') then
        begin
          Params:=Tokenize(TS,'|');
          if(Params[Params.Count-1]=Key) then
          begin
            Result:=Params[0];
            break;
          end;
        end;
      until (TS='[/'+SName+']')  or (i = ListStrings.Count);
      Break;
    end;
    inc(i);
  end; //end of while
end;



destructor TListFile.Destroy;
begin
  inherited Destroy;
  ListStrings.Free;
  if not CreatedFromStream then CloseFile(LFile);
end;

end.

