//     ?   .
//      ""

unit Misc;

interface

uses
  Windows, SysUtils, Classes, bzip2;

function ReplaceStr(const S, Srch, Replace: string): string;
function GetFileSize(Name: string) : integer;
function Tokenize(Str: WideString; Delimiter: string): TStringList;
procedure DoCompressStream(ASource, ADest: TStream);
procedure CompressFile(const ASource, ADest: TFileName);
procedure DoUncompressStream(ASource, ADest: TStream);
procedure DoUncompress(const ASource, ADest: TFileName);
procedure CompressFiles(List: TStrings; OutDir: string);

implementation
var
 FSource: TStream;

procedure CompressFiles(List: TStrings; OutDir: string);
var i: integer;
begin
 for i:=0 to List.Count-1 do
 begin
  CompressFile(List[i],OutDir+ExtractFileName(List[i]));
 end;
end;

function Tokenize(Str: WideString; Delimiter: string): TStringList;
var
  tmpStrList: TStringList;
  tmpString, tmpVal: WideString;
  DelimPos: LongInt;
begin
  tmpStrList := TStringList.Create;
  TmpString := Str;
  DelimPos := 1;
  while DelimPos > 0 do
  begin
    DelimPos := LastDelimiter(Delimiter, TmpString);
    tmpVal := Copy(TmpString, DelimPos + 1, Length(TmpString));
    if tmpVal <> '' then
      tmpStrList.Add(tmpVal);
    Delete(TmpString, DelimPos, Length(TmpString));
  end;
  Tokenize := tmpStrList;
end;


//  ...
function GetFileSize(Name: String): Integer;
var
  FStream: TFileStream;
begin
  if(FileExists(Name)) then
    FStream:=TFileStream.Create(Name, fmOpenRead)
  else Result:=-1;
  if Result<>-1 then
  begin
    Result:=FStream.Size;
    FStream.Free;
  end;
end;

// -    
function ReplaceStr(const S, Srch, Replace: string): string;
var
  I: Integer;
  Source: string;
begin
  Source := S;
  Result := '';
  repeat
    I := Pos(Srch, Source);
    if I>0 then
    begin
      Result := Result + Copy(Source, 1, I - 1) + Replace;
      Source := Copy(Source, I + Length(Srch), MaxInt);
    end
    else
      Result := Result + Source;
  until I<= 0;
end;

procedure DoCompressStream(ASource, ADest: TStream);
var
  Comp: TBZCompressionStream;
begin
  FSource := ASource;
  Comp := TBZCompressionStream.Create(bs9, ADest);
  try
    Comp.CopyFrom(ASource, 0);
  finally
    Comp.Free;
    FSource := nil;
  end;
end;


procedure CompressFile(const ASource, ADest: TFileName);
var
  Source, Dest: TStream;
begin
  Source := TFileStream.Create(ASource, fmOpenRead + fmShareDenyWrite);
  try
    Dest := TFileStream.Create(ADest, fmCreate);
    try
      DoCompressStream(Source, Dest);
    finally
      Dest.Free;
    end;
  finally
    Source.Free;
  end;
end;

procedure DoUncompress(const ASource, ADest: TFileName);
var
  Source, Dest: TStream;
begin
  Source := TFileStream.Create(ASource, fmOpenRead + fmShareDenyWrite);
  try
    Dest := TFileStream.Create(ADest, fmCreate);
    try
      DoUncompressStream(Source, Dest);
    finally
      Dest.Free;
    end;
  finally
    Source.Free;
    DeleteFile(ASource);
  end;
end;

procedure DoUncompressStream(ASource, ADest: TStream);
const
  BufferSize = 65536;
var
  Count: Integer;
  Decomp: TBZDecompressionStream;
  Buffer: array[0..BufferSize - 1] of Byte;
begin
  FSource := ASource;
  Decomp := TBZDecompressionStream.Create(ASource);
  try
    while True do
    begin
      Count := Decomp.Read(Buffer, BufferSize);
      if Count <> 0 then ADest.WriteBuffer(Buffer, Count) else Break;
    end;
  finally
    Decomp.Free;
    FSource := nil;
  end;
end;

end.
