unit GetFilesThr;

interface

uses
  Classes, Wininet, Windows, SysUtils, Messages, Dialogs, Forms;

const
 UPD_MESSAGE = WM_USER + 1;
 UPD_PROGRESS_MAX = WM_USER + 2;
 UPD_PROGRESS = WM_USER + 3;
 UPD_LOCK_FORM = WM_USER + 4;
 UPD_UPDATE_SET = WM_USER + 5; 

type
  GFilesThread = class(TThread)
  private
    FilesToGet : TStringList;     //   
    FilesSize : Longword;         //    
    CBackup : integer;            //   
    CRevision : integer;          //  
    CForceCheck : boolean;        //    [          ]
    UUrl : string;                // URL   
    USelfParam : string;          //   ,   - 
    USelfSkn : string;
    Dir: string;                  // ""  [     ]
    FSource: TStream;             //  
  protected
    procedure Execute; override;
    procedure CheckFiles(FList : TStringList);
    procedure BZProgress(Sender: TObject);
    procedure GetFiles;
    procedure SelfUpdate(SelfVal, AppExe : string);
    procedure UpdateSkn(SelfVal, AppExe : string);
    procedure ModHosts(Lines : TStringList);
    procedure DoUncompressStream(ASource, ADest: TStream);
    procedure DoUncompress(const ASource, ADest: TFileName);
    function HTTPGetFile(const fileURL, FileName: string; sh_progress: boolean): boolean;
  public
    property CreateBackup : integer write CBackup;
    property UpdatesUrl : string write UUrl;
    property LocalRevision : integer write CRevision;
    property ForceCheck : boolean write CForceCheck;
    property WorkDir: string write Dir;
  end;

implementation
uses Unit1, Misc, BZip2, md5, LstFile;

var LTemp, SknUpdateFlag, SelfUpdateFlag: cardinal;


procedure GFilesThread.BZProgress(Sender: TObject);
begin
  SendMessage(Form1.Handle, UPD_PROGRESS,0,FSource.Position);
end;

function GFilesThread.HTTPGetFile(const fileURL, FileName: string; sh_progress: boolean): boolean;
const
  BufferSize = 1024;
var
  hSession, hURL: HInternet;
  Buffer: array[1..BufferSize] of Byte;
  BufferLen: Longword;
  f: file;
  sAppName: string;
begin
  Result := False;
  sAppName := 'L3UpdaterClient';
  hSession := InternetOpen(PChar(sAppName),
  INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);
  try
    hURL := InternetOpenURL(hSession, PChar(fileURL), nil, 0, 0, 0);
    if (hURL <> nil) then  begin
    try
      DeleteUrlCacheEntry(PChar(fileURL));
      AssignFile(f, FileName);
      Rewrite(f,1);
      repeat
        InternetReadFile(hURL, @Buffer, SizeOf(Buffer), BufferLen);
        BlockWrite(f, Buffer, BufferLen);
        if (sh_progress) then
        begin
          LTemp:=LTemp+BufferLen;
          SendMessage(Form1.Handle, UPD_PROGRESS,0,LTemp+BufferLen);
        end;
      until
        BufferLen = 0;
      CloseFile(f);
      Result := True;
    finally
      InternetCloseHandle(hURL);
    end;
  end;
  finally
    InternetCloseHandle(hSession);
  end;
  LTemp:=0;
  SendMessage(Form1.Handle, UPD_PROGRESS,0,0);
end;

procedure GFilesThread.DoUncompress(const ASource, ADest: TFileName);
var
  Source, Dest: TStream;
begin
  Source := TFileStream.Create(ASource, fmOpenRead + fmShareDenyWrite);
  try
    Dest := TFileStream.Create(ADest, fmCreate);
    try
      DoUncompressStream(Source, Dest);
    finally
      Dest.Free;
    end;
  finally
    Source.Free;
    DeleteFile(ASource);
    SendMessage(Form1.Handle, UPD_PROGRESS,0,0);
  end;
end;

procedure GFilesThread.DoUncompressStream(ASource, ADest: TStream);
const
  BufferSize = 65536;
var
  Count: Integer;
  Decomp: TBZDecompressionStream;
  Buffer: array[0..BufferSize - 1] of Byte;
begin
  FSource := ASource;
  SendMessage(Form1.Handle, UPD_PROGRESS_MAX,0,FSource.Size);
  Decomp := TBZDecompressionStream.Create(ASource);
  try
    Decomp.OnProgress := BZProgress;
    while True do
    begin
      Count := Decomp.Read(Buffer, BufferSize);
      if Count <> 0 then ADest.WriteBuffer(Buffer, Count) else Break;
    end;
  finally
    Decomp.Free;
    FSource := nil;
  end;
end;


procedure GFilesThread.CheckFiles(FList : TStringList);
var
  i: integer;
  FParam: TStringList;
  FNameLocal: string;
begin
  if(FList.Count>0) and (FList[0]<>'FAIL') and (not terminated) then
  begin
    SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('Checking files')));
    SendMessage(Form1.Handle, UPD_PROGRESS_MAX,1,FList.Count-1);
    FParam:=TStringList.Create;
    for i:=0 to FList.Count-1 do
    begin
      SendMessage(Form1.Handle, UPD_PROGRESS,1,i);
      FParam:=Tokenize(FList[i],'|');
      FNameLocal:=Dir+FParam[2];
        SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('Checking '+FParam[2])));
      if (not FileExists(FNameLocal)) then
      begin
        FilesToGet.Add(FList[i]);
        FilesSize:=FilesSize+StrToInt(FParam[0]);
      end
      else
      begin
        if (MD5Print(MD5File(FNameLocal))<>FParam[1]) then
        begin
          FilesToGet.Add(FList[i]);
          FilesSize:=FilesSize+StrToInt(FParam[0]);
        end;
      end;
    end;
    FParam.Free;
    SendMessage(Form1.Handle, UPD_PROGRESS,1,0);
    SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('')));
  end;
end;

procedure GFilesThread.SelfUpdate(SelfVal, AppExe : string);
var
  FParam: TStringList;
  FNameLocal: string;
  F:boolean;
begin
  SelfUpdateFlag:=0;
  if(SelfVal<>'') then
  begin
    FParam:=TStringList.Create;
    FParam:=Tokenize(SelfVal,'|');
      FNameLocal:=Dir+FParam[2];
      if (MD5Print(MD5File(FNameLocal))<>FParam[1]) and (AppExe = FParam[2]) then
      begin
        FilesSize:=FilesSize+StrToInt(FParam[0]);
        F:=HTTPGetFile(UUrl+FParam[2]+'.bz2',FNameLocal+'.bz2',True);
        if(F) then begin
          try
           DoUncompress(FNameLocal+'.bz2',Dir+FParam[2]+'.New');
           //GenKillerBat(FParam[2]);
           SelfUpdateFlag:=1;//RunApp(Dir+'Update.bat');
          except
            SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('Self Update Failed')));
            DeleteFile(FNameLocal);
          end;
        end;
      end;
    FParam.Free;
  end;
end;

procedure GFilesThread.UpdateSkn(SelfVal, AppExe : string);
var
  FParam: TStringList;
  FNameLocal: string;
  F:boolean;
begin
  SknUpdateFlag:=0;
  if(SelfVal<>'') then
  begin
    FParam:=TStringList.Create;
    FParam:=Tokenize(SelfVal,'|');
      FNameLocal:=Dir+FParam[2];
      if (MD5Print(MD5File(FNameLocal))<>FParam[1]) and (AppExe = ChangeFileExt(FParam[2],'.l3s')) then
      begin
        FilesSize:=FilesSize+StrToInt(FParam[0]);
        F:=HTTPGetFile(UUrl+FParam[2]+'.bz2',FNameLocal+'.bz2',True);
        if(F) then begin
          try
           DoUncompress(FNameLocal+'.bz2',Dir+FParam[2]);
           //ReBat(AppExe);
           SknUpdateFlag:=1;//RunApp(Dir+'Update.bat');
          except
            SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('Settings Update Failed')));
            DeleteFile(FNameLocal);
          end;
        end;
      end;
    FParam.Free;
  end;
end;

procedure GFilesThread.ModHosts(Lines : TStringList);
var
 Hosts : textfile;
 H, HostsStrings, HostLineParam : TStringList;
 HostsPath, temp : string;
 i, z, funnyFlag : integer;
 WindirP : PChar;
 Res : cardinal;
begin
  WinDirP := StrAlloc(MAX_PATH);
  Res := GetWindowsDirectory(WinDirP, MAX_PATH);
  if Res > 0 then
  begin
    //     MSN ,   hosts.msn
    if(FileExists(StrPas(WinDirP)+'\system32\drivers\etc\hosts.msn')) then
      HostsPath := StrPas(WinDirP)+'\system32\drivers\etc\hosts.msn'
    else
      HostsPath := StrPas(WinDirP)+'\system32\drivers\etc\hosts';
    AssignFile(Hosts,HostsPath);
    Reset(Hosts);
    HostsStrings:= TStringList.Create;
    H:= TStringList.Create;
    H.Add('#-------- Added by L2Updater --------');
    //     \
    while (not Eof(Hosts)) do
    begin
      ReadLn(Hosts, temp);
      HostsStrings.Add(Trim(temp));
    end ;
    Reset(Hosts);
    for i:=0 to Lines.Count-1 do
    begin
      funnyFlag:=0;
      HostLineParam:=Tokenize(Lines[i],'|');
      for z:=0 to HostsStrings.Count-1 do
      begin
       if (StrSearch(1,HostsStrings[z],HostLineParam[0])>0) and (HostsStrings[z][1]<>'#') then
       begin
          if (StrSearch(1,HostsStrings[z],HostLineParam[1]+#9)= 0) and (StrSearch(1,HostsStrings[z],HostLineParam[1]+' ')= 0 ) then
          begin
           HostsStrings[z]:= '#'+HostsStrings[z];
           funnyFlag:=1;
          end
          else funnyFlag:=2;
       end;
      end;
      if (funnyFlag=1) or (funnyFlag=0)  then
        H.Add(HostLineParam[1]+#9+HostLineParam[0]);
    end;
    H.Add('#-----------------');
    if H.Count>2 then
    begin
      Rewrite(Hosts);
        SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('Applying changes to Hosts')));
      for i:=0 to HostsStrings.Count-1 do
      begin
        WriteLn(Hosts,HostsStrings[i]);
      end;

      for i:=0 to H.Count-1 do
      begin
       WriteLn(Hosts,H[i]);
      end;
        SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('Hosts file changed')));
    end;
      H.Free; HostsStrings.Free; HostLineParam.Free;
  CloseFile(Hosts);
  end;
end;

procedure GFilesThread.GetFiles;
var
  FParam : TStringList;
  i : integer;
  F,  error : boolean;
  LocalFile, BakFile: string;
begin
  error := False;
  if (FilesToGet.Count>0) then
  begin
    FParam:=TStringList.Create;
    SendMessage(Form1.Handle, UPD_PROGRESS_MAX,1,FilesToGet.Count-1);
    i:=0;
    while (i < FilesToGet.Count) and (not terminated) do
    begin
//       
      FParam:=Tokenize(FilesToGet[i],'|');
      LocalFile:= Dir+FParam[2];

      SendMessage(Form1.Handle, UPD_MESSAGE, 0,
          DWORD(PChar('Downloading '+ FParam[2])));

//       .     
      SendMessage(Form1.Handle, UPD_PROGRESS_MAX,0,StrToInt(FParam[0]));

      if (not DirectoryExists(ExtractFilePath(LocalFile))) then
        ForceDirectories(ExtractFilePath(LocalFile));
      F:=HTTPGetFile(UUrl+ReplaceStr(FParam[2],'\','/')+'.bz2',LocalFile+'.bz2',True);
      if (F) then
      begin
        try
          if (CBackup=1) then
          begin
            BakFile:=Dir+'backup\'+FParam[2];
            if (not DirectoryExists(ExtractFilePath(BakFile))) then
              ForceDirectories(ExtractFilePath(BakFile));
            CopyFile(PChar(LocalFile),PChar(BakFile),false);
          end;
          SendMessage(Form1.Handle, UPD_MESSAGE, 0,
            DWORD(PChar('Extracting '+ FParam[2])));
          DoUncompress(LocalFile+'.bz2',Dir+FParam[2]);
        except
          SendMessage(Form1.Handle, UPD_MESSAGE, 0,
            DWORD(PChar('Update Failed')));
          error := True;
        end;
      end
      else
      begin
          SendMessage(Form1.Handle, UPD_MESSAGE, 0,
            DWORD(PChar('Update Failed')));
        error := True;
        Break;
      end;
    inc(i);
    SendMessage(Form1.Handle, UPD_PROGRESS,1,i);
  end;
  FParam.Free;
  if (not error) then
   SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('All the files have been updated')));
  end
  else SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('')));
  SendMessage(Form1.Handle, UPD_PROGRESS,1,0);
end;

procedure GFilesThread.Execute;
var
  List: TListFile;
  CFiles, NFiles, TempList, HostsLines : TStringList;
  TRev, IsModHosts : integer;
  F : boolean;
  Name : string;
begin
  FilesSize:=0;
  TempList:=TStringList.Create;
  SendMessage(Form1.Handle, UPD_LOCK_FORM, 0, 0);
  SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('Downloading updates list')));
  if(UUrl[length(UUrl)]<>'/') then UUrl:=UUrl+'/';
  F:=HTTPGetFile(UUrl+'files.lst.bz2',Dir+'files.lst.bz2', True);
  if (F) then
  begin
   SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('')));
   try
      DoUncompress(Dir+'files.lst.bz2',Dir+'files.lst');
    except
      SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('Update Failed')));
      DeleteFile(Dir+'files.lst');
    end;
    if(FileExists(Dir+'files.lst')) then
    begin
      FilesToGet := TStringList.Create;
      List := TListFile.Create(Dir+'files.lst');
      CFiles:=TStringList.Create;
      TRev:=StrToInt(List.GetKeyValue('settings','Rev'));
      IsModHosts:=StrToInt(List.GetKeyValue('settings','ModHosts'));
      if (IsModHosts = 1) then
      begin
        HostsLines:= TStringList.Create;
        HostsLines:= List.GetSection('hosts');
        try
          ModHosts(HostsLines);
        finally
          HostsLines.Free;
        end;
      end;
      TempList:= List.GetSection('self');
      USelfParam:=TempList[0];
      Name:=ExtractFileName(Application.ExeName);
      if TempList.Count>1 then
      begin
        USelfSkn:=TempList[1];
        UpdateSkn(USelfSkn,Name);
      end;
      if(TempList[0]<>'FAIL') then SelfUpdate(USelfParam,Name); //    :)
      TempList.Free;

      if (SelfUpdateFlag=1) and (SknUpdateFlag=0) then
      begin
        GenKillerBat(Name);
      end;
      if (SknUpdateFlag=1) and (SelfUpdateFlag=0) then
      begin
          ReBat(Name);
      end;
      if (SelfUpdateFlag=1) and (SknUpdateFlag=1) then
      begin
        GenKillerBat(Name);
      end;


      CFiles:=List.GetSection('files_critical');
      CheckFiles(CFiles); //   
      CFiles.Free;
      if (CForceCheck) or (TRev>CRevision) then //      ,   
      begin
        if (CBackup=1) then
        begin
          DelDir(Dir+'backup');
          MkDir(Dir+'backup');
        end;
        NFiles:=TStringList.Create;
        NFiles:=List.GetSection('files_normal');
        CheckFiles(NFiles);
        NFiles.Free;
      end;
      GetFiles;
      List.Destroy;
      FilesToGet.Free;
      DeleteFile(Dir+'files.lst');
      if TRev>CRevision then
      begin
        SendMessage(Form1.Handle, UPD_UPDATE_SET, 0, TRev);
      end;
    end;
  end
  else
  begin
    SendMessage(Form1.Handle, UPD_MESSAGE, 0, DWORD(PChar('Update Failed')));
    DeleteFile(Dir+'files.lst');
  end;
  SendMessage(Form1.Handle, UPD_LOCK_FORM, 0, 1);
end;

end.
